@@ -7,7 +7,7 @@ use Params::Validate qw( validate SCALAR BOOLEAN OBJECT CODEREF ARRAYREF );
 use DateTime 0.12;  # this is for version checking only
 use DateTime::Duration;
 use DateTime::Span;
-use Set::Infinite 0.5503;
+use Set::Infinite 0.59;
 use Set::Infinite::_recurrence;
 
 use vars qw( $VERSION );
@@ -16,10 +16,49 @@ use constant INFINITY     =>       100 ** 100 ** 100 ;
 use constant NEG_INFINITY => -1 * (100 ** 100 ** 100);
 
 BEGIN {
-    $VERSION = '0.16';
+    $VERSION = '0.1603';
+}
+
+
+sub _fix_datetime {
+    # internal function -
+    # (not a class method)
+    #
+    # checks that the parameter is an object, and
+    # also protects the object against mutation
+    
+    return $_[0]
+        unless defined $_[0];      # error
+    return $_[0]->clone
+        if ref( $_[0] );           # "immutable" datetime
+    return DateTime::Infinite::Future->new 
+        if $_[0] == INFINITY;      # Inf
+    return DateTime::Infinite::Past->new
+        if $_[0] == NEG_INFINITY;  # -Inf
+    return $_[0];                  # error
+}
+
+sub _fix_return_datetime {
+    my ( $dt, $dt_arg ) = @_;
+
+    # internal function -
+    # (not a class method)
+    #
+    # checks that the returned datetime has the same
+    # time zone as the parameter
+
+    # TODO: set locale
+
+    if ( $dt_arg->can('time_zone_long_name') &&
+         !( $dt_arg->time_zone_long_name eq 'floating' ) )
+    {
+        $dt->set_time_zone( $dt_arg->time_zone );
+    }
+    return $dt;
 }
 
 sub iterate {
+    # deprecated method - use map() or grep() instead
     my ( $self, $callback ) = @_;
     my $class = ref( $self );
     my $return = $class->empty_set;
@@ -80,16 +119,54 @@ sub subtract_duration { return $_[0]->add_duration( $_[1]->inverse ) }
 sub add_duration {
     my ( $self, $dur ) = @_;
     $dur = $dur->clone;  # $dur must be "immutable"
-    return $self->iterate(
-        sub { $_[0]->add_duration( $dur ) }
+
+    $self->{set} = $self->{set}->iterate(
+        sub {
+            my $min = $_[0]->min;
+            $min->clone->add_duration( $dur ) if ref($min);
+        },
+        backtrack_callback => sub { 
+            my ( $min, $max ) = ( $_[0]->min, $_[0]->max );
+            if ( ref($min) )
+            {
+                $min = $min->clone;
+                $min->subtract_duration( $dur );
+            }
+            if ( ref($max) )
+            {
+                $max = $max->clone;
+                $max->subtract_duration( $dur );
+            }
+            return Set::Infinite::_recurrence->new( $min, $max );
+        },
     );
+    $self;
 }
 
 sub set_time_zone {
     my ( $self, $tz ) = @_;
-    return $self->iterate( 
-        sub { $_[0]->set_time_zone( $tz ) }
+
+    $self->{set} = $self->{set}->iterate(
+        sub {
+            my $min = $_[0]->min;
+            $min->clone->set_time_zone( $tz ) if ref($min);
+        },
+        backtrack_callback => sub {
+            my ( $min, $max ) = ( $_[0]->min, $_[0]->max );
+            if ( ref($min) )
+            {
+                $min = $min->clone;
+                $min->set_time_zone( $tz );
+            }
+            if ( ref($max) )
+            {
+                $max = $max->clone;
+                $max->set_time_zone( $tz );
+            }
+            return Set::Infinite::_recurrence->new( $min, $max );
+        },
     );
+    $self;
 }
 
 sub set {
@@ -99,138 +176,101 @@ sub set {
                                        default => undef },
                          }
                        );
-    return $self->iterate( 
-        sub { $_[0]->set( %args ) }
+    $self->{set} = $self->{set}->iterate( 
+        sub {
+            my $min = $_[0]->min;
+            $min->clone->set( %args ) if ref($min);
+        },
     );
+    $self;
 }
 
 sub from_recurrence {
     my $class = shift;
-    # note: not using validate() because this is too complex...
+
     my %args = @_;
     my %param;
+    
     # Parameter renaming, such that we can use either
     #   recurrence => xxx   or   next => xxx, previous => xxx
-    $param{next} = delete $args{recurrence}  or
-    $param{next} = delete $args{next};
-    $param{previous} = $args{previous}  and  delete $args{previous};
+    $param{next} = delete $args{recurrence} || delete $args{next};
+    $param{previous} = delete $args{previous};
 
     ## $param{detect_bounded} = delete $args{detect_bounded} or 0;
 
-    $param{span} = $args{span}  and  delete $args{span};
+    $param{span} = delete $args{span};
     # they might be specifying a span using begin / end
     $param{span} = DateTime::Span->new( %args ) if keys %args;
 
     my $self = {};
-    if ($param{next} || $param{previous}) 
+    
+    die "Not enough arguments in from_recurrence()"
+        unless $param{next} || $param{previous}; 
+
+    if ( ! $param{previous} ) 
     {
-
-        if ( ! $param{previous} ) 
-        {
-            my $data = {};
-            $param{previous} =
-                sub {
-                    # "objectify" infinity
-                    if ( ! ref( $_[0] ) )
-                    {
-                        if ( $_[0] == NEG_INFINITY ) {
-                            $_[0] = DateTime::Infinite::Past->new; 
-                        }
-                        elsif ( $_[0] == INFINITY ) {
-                            $_[0] = DateTime::Infinite::Future->new 
-                        }
-                    }
-                    _callback_previous ( $_[0], $param{next}, $data );
-                }
-        }
-        else
-        {
-            my $previous = $param{previous};
-            $param{previous} =
-                sub {
-                    # "objectify" infinity
-                    if ( ! ref( $_[0] ) )
-                    {
-                        if ( $_[0] == NEG_INFINITY ) {
-                            $_[0] = DateTime::Infinite::Past->new; 
-                        }
-                        elsif ( $_[0] == INFINITY ) {
-                            $_[0] = DateTime::Infinite::Future->new 
-                        }
-                    }
-                    $previous->( $_[0]->clone );
-                }
-        }
-
-        if ( ! $param{next} ) 
-        {
-            my $data = {};
-            $param{next} =
-                sub {
-                    # "objectify" infinity
-                    if ( ! ref( $_[0] ) )
-                    {
-                        if ( $_[0] == NEG_INFINITY ) {
-                            $_[0] = DateTime::Infinite::Past->new; 
-                        }
-                        elsif ( $_[0] == INFINITY ) {
-                            $_[0] = DateTime::Infinite::Future->new 
-                        }
-                    }
-                    _callback_next ( $_[0], $param{previous}, $data );
-                }
-        }
-        else
-        {
-            my $next = $param{next};
-            $param{next} =
-                sub {
-                    # "objectify" infinity
-                    if ( ! ref( $_[0] ) )
-                    {
-                        if ( $_[0] == NEG_INFINITY ) {
-                            $_[0] = DateTime::Infinite::Past->new; 
-                        }
-                        elsif ( $_[0] == INFINITY ) {
-                            $_[0] = DateTime::Infinite::Future->new 
-                        }
-                    }
-                    $next->( $_[0]->clone );
-                }
-        }
-
-        my ( $min, $max );
-        ## if ( $param{detect_bounded} )
-        {
-            $max = $param{previous}->( DateTime::Infinite::Future->new );
-            $min = $param{next}->( DateTime::Infinite::Past->new );
-            $max = INFINITY if $max->is_infinite;
-            $min = NEG_INFINITY if $min->is_infinite;
-        }
-        
-        # else
-        # {
-        #    $max = INFINITY;
-        #    $min = NEG_INFINITY;
-        # }
-        
-        my $base_set = Set::Infinite::_recurrence->new( $min, $max );
-        $base_set = $base_set->intersection( $param{span}->{set} )
-             if $param{span};
-        # warn "base set is $base_set\n";
-
         my $data = {};
-        $self->{set} = 
+        $param{previous} =
+                sub {
+                    _callback_previous ( _fix_datetime( $_[0] ), $param{next}, $data );
+                }
+    }
+    else
+    {
+        my $previous = $param{previous};
+        $param{previous} =
+                sub {
+                    $previous->( _fix_datetime( $_[0] ) );
+                }
+    }
+
+    if ( ! $param{next} ) 
+    {
+        my $data = {};
+        $param{next} =
+                sub {
+                    _callback_next ( _fix_datetime( $_[0] ), $param{previous}, $data );
+                }
+    }
+    else
+    {
+        my $next = $param{next};
+        $param{next} =
+                sub {
+                    $next->( _fix_datetime( $_[0] ) );
+                }
+    }
+
+    my ( $min, $max );
+    ## if ( $param{detect_bounded} )
+    {
+        $max = $param{previous}->( DateTime::Infinite::Future->new );
+        $min = $param{next}->( DateTime::Infinite::Past->new );
+        $max = INFINITY if $max->is_infinite;
+        $min = NEG_INFINITY if $min->is_infinite;
+    }
+        
+    # else
+    # {
+    #    $max = INFINITY;
+    #    $min = NEG_INFINITY;
+    # }
+        
+    my $base_set = Set::Infinite::_recurrence->new( $min, $max );
+    $base_set = $base_set->intersection( $param{span}->{set} )
+         if $param{span};
+         
+    # warn "base set is $base_set\n";
+
+    my $data = {};
+    $self->{set} = 
             $base_set->_recurrence(
                 $param{next}, 
                 $param{previous},
                 $data,
-            );
-        bless $self, $class;
-    }
-    else {
-        die "Not enough arguments in from_recurrence()";
-    }
+        );
+    bless $self, $class;
+    
     return $self;
 }
 
@@ -244,9 +284,16 @@ sub from_datetimes {
                        );
     my $self = {};
     $self->{set} = Set::Infinite::_recurrence->new;
-    # possible optimization: sort dates and use "push"
+    # possible optimization: sort datetimes and use "push"
     for( @{ $args{dates} } ) 
     {
+        # DateTime::Infinite objects are not welcome here,
+        # but this is not enforced (it does't hurt)
+
+        carp "The 'dates' argument to from_datetimes() must only contain ".
+             "datetime objects"
+            unless UNIVERSAL::can( $_, 'utc_rd_values' );
+
         $self->{set} = $self->{set}->union( $_->clone );
     }
 
@@ -272,8 +319,6 @@ sub clone {
 # This is used to simulate a 'previous' callback,
 # when then 'previous' argument in 'from_recurrence' is missing.
 #
-use DateTime::Infinite;
-
 sub _callback_previous {
     my ($value, $callback_next, $callback_info) = @_; 
     my $previous = $value->clone;
@@ -377,12 +422,14 @@ sub next {
     {
         if ( $self->{set}->_is_recurrence )
         {
-            return $self->{set}->{param}[0]->( $_[0] );
+            return _fix_return_datetime(
+                       $self->{set}->{param}[0]->( $_[0] ), $_[0] );
         }
         else 
         {
             my $span = DateTime::Span->from_datetimes( after => $_[0] );
-            return $self->intersection( $span )->next;
+            return _fix_return_datetime(
+                        $self->intersection( $span )->next, $_[0] );
         }
     }
 
@@ -402,12 +449,14 @@ sub previous {
     {
         if ( $self->{set}->_is_recurrence ) 
         {
-            return $self->{set}->{param}[1]->( $_[0] );
+            return _fix_return_datetime(
+                      $self->{set}->{param}[1]->( $_[0] ), $_[0] );
         }
         else 
         {
             my $span = DateTime::Span->from_datetimes( before => $_[0] );
-            return $self->intersection( $span )->previous;
+            return _fix_return_datetime(
+                      $self->intersection( $span )->previous, $_[0] );
         }
     }
 
@@ -417,7 +466,7 @@ sub previous {
     return $head;
 }
 
-# "current" means less-or-equal to a DateTime
+# "current" means less-or-equal to a datetime
 sub current {
     my $self = shift;
 
@@ -483,6 +532,8 @@ sub intersection {
     my ($set1, $set2) = ( shift, shift );
     my $class = ref($set1);
     my $tmp = $class->empty_set();
+    $set2 = $set2->as_set
+        if $set2->can( 'as_set' );
     $set2 = $class->from_datetimes( dates => [ $set2, @_ ] ) 
         unless $set2->can( 'union' );
     $tmp->{set} = $set1->{set}->intersection( $set2->{set} );
@@ -492,6 +543,8 @@ sub intersection {
 sub intersects {
     my ($set1, $set2) = ( shift, shift );
     my $class = ref($set1);
+    $set2 = $set2->as_set
+        if $set2->can( 'as_set' );
     unless ( $set2->can( 'union' ) )
     {
         if ( $set1->{set}->_is_recurrence )
@@ -510,6 +563,8 @@ sub intersects {
 sub contains {
     my ($set1, $set2) = ( shift, shift );
     my $class = ref($set1);
+    $set2 = $set2->as_set
+        if $set2->can( 'as_set' );
     unless ( $set2->can( 'union' ) )
     {
         if ( $set1->{set}->_is_recurrence )
@@ -529,6 +584,8 @@ sub union {
     my ($set1, $set2) = ( shift, shift );
     my $class = ref($set1);
     my $tmp = $class->empty_set();
+    $set2 = $set2->as_set
+        if $set2->can( 'as_set' );
     $set2 = $class->from_datetimes( dates => [ $set2, @_ ] ) 
         unless $set2->can( 'union' );
     $tmp->{set} = $set1->{set}->union( $set2->{set} );
@@ -543,6 +600,8 @@ sub complement {
     my $tmp = $class->empty_set();
     if (defined $set2) 
     {
+        $set2 = $set2->as_set
+            if $set2->can( 'as_set' );
         $set2 = $class->from_datetimes( dates => [ $set2, @_ ] ) 
             unless $set2->can( 'union' );
         # TODO: "compose complement";
@@ -557,31 +616,11 @@ sub complement {
 }
 
 sub min { 
-    my $tmp = $_[0]->{set}->min;
-    if ( ref($tmp) ) 
-    {
-        $tmp = $tmp->clone;
-    } 
-    else
-    {
-        $tmp = new DateTime::Infinite::Past 
-            if defined $tmp && $tmp == NEG_INFINITY;
-    }
-    $tmp;
+    return _fix_datetime( $_[0]->{set}->min );
 }
 
 sub max { 
-    my $tmp = $_[0]->{set}->max;
-    if ( ref($tmp) ) 
-    {
-        $tmp = $tmp->clone;
-    } 
-    else
-    {
-        $tmp = new DateTime::Infinite::Future 
-            if defined $tmp && $tmp == INFINITY;
-    }
-    $tmp;
+    return _fix_datetime( $_[0]->{set}->max );
 }
 
 # returns a DateTime::Span
@@ -661,11 +700,11 @@ DateTime::Set - Datetime sets and set math
 
 =head1 DESCRIPTION
 
-DateTime::Set is a module for date/time sets.  It can be used to
+DateTime::Set is a module for datetime sets.  It can be used to
 handle two different types of sets.
 
 The first is a fixed set of predefined datetime objects.  For example,
-if we wanted to create a set of dates containing the birthdays of
+if we wanted to create a set of datetimes containing the birthdays of
 people in our family.
 
 The second type of set that it can handle is one based on the idea of
@@ -682,10 +721,16 @@ time", or "every Wednesday after 2003-03-05 until the end of time", or
 
 =item * from_datetimes
 
-Creates a new set from a list of dates.
+Creates a new set from a list of datetimes.
 
    $dates = DateTime::Set->from_datetimes( dates => [ $dt1, $dt2, $dt3 ] );
 
+The datetimes can be objects from class C<DateTime>, or from a
+C<DateTime::Calendar::*> class.
+
+C<DateTime::Infinite::*> objects are not valid set members. 
+However, these datetimes are very useful as set boundaries.
+
 =item * from_recurrence
 
 Creates a new set specified via a "recurrence" callback.
@@ -693,7 +738,6 @@ Creates a new set specified via a "recurrence" callback.
     $months = DateTime::Set->from_recurrence( 
         span => $dt_span_this_year,    # optional span
         recurrence => sub { 
-            return $_[0] if $_[0]->is_infinite;
             return $_[0]->truncate( to => 'month' )->add( months => 1 ) 
         }, 
     );
@@ -707,48 +751,46 @@ case, if there is a C<span> parameter it will be ignored.
     $months = DateTime::Set->from_recurrence(
         after => $dt_now,
         recurrence => sub {
-            return $_[0] if $_[0]->is_infinite;
             return $_[0]->truncate( to => 'month' )->add( months => 1 );
         },
     );
 
-The recurrence will be passed a single parameter, a DateTime.pm
-object.  The recurrence must return the I<next> event 
-after that object.  There is no guarantee as to what the object will
-be set to, only that it will be greater than the object
+The recurrence function will be passed a single parameter, a datetime
+object. The parameter can be an object from class C<DateTime>, 
+or from one of the C<DateTime::Calendar::*> classes. 
+The parameter can also be a C<DateTime::Infinite::Future> or
+a C<DateTime::Infinite::Past> object.
+
+The recurrence must return the I<next> event 
+after that object.  There is no guarantee as to what the returned 
+object will be set to, only that it will be greater than the object
 passed to the recurrence.
+If there are no more datetimes after the given parameter, 
+then the recurrence function should return C<DateTime::Infinite::Future>.
 
-The recurrence function must return a valid DateTime object.
-
-The function must work if given C<DateTime::Infinite::Future> and 
-C<DateTime::Infinite::Past> parameters.
-
-It is ok to modify C<$_[0]> inside the recurrence function.
+It is ok to modify the parameter C<$_[0]> inside the recurrence function.
 There are no side-effects.
 
 For example, if you wanted a recurrence that generated datetimes in
-increments of 30 seconds would look like this:
+increments of 30 seconds, it would look like this:
 
   sub every_30_seconds {
       my $dt = shift;
-      return $dt if $dt->is_infinite;
-
-      $dt->truncate( to => 'seconds' );
-
       if ( $dt->second < 30 ) {
-          return $dt->add( seconds => 30 - $dt->second );
+          return $dt->truncate( to => 'minute' )->add( seconds => 30 );
       } else {
-          return $dt->add( seconds => 60 - $dt->second );
+          return $dt->truncate( to => 'minute' )->add( minutes => 1 );
       }
   }
 
-Of course, this recurrence ignores leap seconds, but we'll leave that
-as an exercise for the reader ;)
+Note that this recurrence takes leap seconds into account. 
+You should use datetime calendar methods whenever possible, 
+in order to avoid complicated arithmetic problems!
 
 It is also possible to create a recurrence by specifying either or both
-'next' and 'previous' callbacks.
+of 'next' and 'previous' callbacks.
 
-Callbacks can return C<DateTime::Infinite::Future> and 
+The callbacks can return C<DateTime::Infinite::Future> and 
 C<DateTime::Infinite::Past> objects, in order to define I<bounded recurrences>.
 In this case, both 'next' and 'previous' callbacks must be defined:
 
@@ -770,21 +812,11 @@ In this case, both 'next' and 'previous' callbacks must be defined:
         },
     );
 
-Bounded recurrences are is easier to write using span parameters:
-
-    # "monthly from $dt until forever"
-
-    $months = DateTime::Set->from_recurrence(
-        start => $dt,
-        recurrence => sub {
-            return $_[0] if $_[0]->is_infinite;
-            return $_[0]->truncate( to => 'month' )->add( months => 1 );
-        },
-    );
+Bounded recurrences are easier to write using C<span> parameters. See above.
 
 See also C<DateTime::Event::Recurrence> and the other C<DateTime::Event::*>
-modules for generating specialized recurrences, such as sunrise and sunset
-time, and holidays.
+factory modules for generating specialized recurrences, 
+such as sunrise and sunset times, and holidays.
 
 =item * empty_set
 
@@ -805,20 +837,14 @@ but you want to keep the previous value:
 
 =item * add_duration( $duration )
 
-This method adds the specified duration added to every element of the set.
+This method adds the specified duration to every element of the set.
 
-    $dtd = new DateTime::Duration( year => 1 );
-    $set->add_duration( $dtd );
+    $dt_dur = new DateTime::Duration( year => 1 );
+    $set->add_duration( $dt_dur );
 
-The original set is not modified. The method returns a new set object.
+The original set is modified. If you want to keep the old values use:
 
-Note: The result of adding a duration to a given set element 
-is expected to be within the span of the
-C<previous> and the C<next> element in the original set.
-
-For example: given the set C<[ 2001, 2010, 2015 ]>,
-the add_duration result for the value C<2010> is expected to be
-within the span C<[ 2001 .. 2015 ]>.
+    $new_set = $set->clone->add_duration( $dt_dur );
 
 =item * add
 
@@ -832,8 +858,6 @@ When given a C<DateTime::Duration> object, this method simply calls
 C<invert()> on that object and passes that new duration to the
 C<add_duration> method.
 
-The original set is not modified. The method returns a new set object.
-
 =item * subtract( DateTime::Duration->new parameters )
 
 Like C<add()>, this is syntactic sugar for the C<subtract_duration()>
@@ -841,31 +865,21 @@ method.
 
 =item * set_time_zone( $tz )
 
-This method accepts either a time zone object or a string that can be
-passed as the "name" parameter to C<< DateTime::TimeZone->new() >>.
-If the new time zone's offset is different from the old time zone,
-then the I<local> time is adjusted accordingly.
-
-If the old time zone was a floating time zone, then no adjustments to
-the local time are made, except to account for leap seconds.  If the
-new time zone is floating, then the I<UTC> time is adjusted in order
-to leave the local time untouched.
-
-The original set C<time zone> is not modified. 
-The method returns a new set object.
+This method will attempt to apply the C<set_time_zone> method to every 
+datetime in the set.
 
 =item * set( locale => .. )
 
-This method can be used to change the C<locale> of a date time set.
+This method can be used to change the C<locale> of a datetime set.
 
-The original set C<locale> is not modified. 
-The method returns a new set object.
+=item * min
 
-=item * min / max
+=item * max
 
-The first and last dates in the set.  These methods may return
+The first and last datetimes in the set.  These methods may return
 C<undef> if the set is empty.  It is also possible that these methods
-may return a C<DateTime::Infinite::Past> or C<DateTime::Infinite::Future> object.
+may return a C<DateTime::Infinite::Past> or C<DateTime::Infinite::Future> 
+object.
 
 =item * span
 
@@ -873,7 +887,7 @@ Returns the total span of the set, as a C<DateTime::Span> object.
 
 =item * iterator / next / previous
 
-These methods can be used to iterate over the dates in a set.
+These methods can be used to iterate over the datetimes in a set.
 
     $iter = $set1->iterator;
     while ( $dt = $iter->next ) {
@@ -932,7 +946,11 @@ not a scalar C<zero>, since a zero count is a valid return value for
 empty sets!
 
 
-=item * union / intersection / complement
+=item * union
+
+=item * intersection 
+
+=item * complement
 
 These set operation methods can accept a C<DateTime> list, 
 a C<DateTime::Set>, a C<DateTime::Span>, or a C<DateTime::SpanSet> 
@@ -949,11 +967,14 @@ C<DateTime::SpanSet> object returns a C<DateTime::SpanSet> object.
 If C<complement> is called without any arguments, then the result is a
 C<DateTime::SpanSet> object representing the spans between each of the
 set's elements.  If complement is given an argument, then the return
-value is a C<DateTime::Set> object.
+value is a C<DateTime::Set> object representing the
+I<set difference> between the sets.
 
 All other operations will always return a C<DateTime::Set>.
 
-=item * intersects / contains
+=item * intersects
+
+=item * contains
 
 These set operations result in a boolean value.
 
@@ -963,7 +984,13 @@ These set operations result in a boolean value.
 These methods can accept a C<DateTime> list, a C<DateTime::Set>,
 a C<DateTime::Span>, or a C<DateTime::SpanSet> object as an argument.
 
-=item * previous / next / current / closest
+=item * previous
+
+=item * next
+
+=item * current
+
+=item * closest
 
   my $dt = $set->next( $dt );
   my $dt = $set->previous( $dt );
@@ -982,6 +1009,10 @@ it returns the closest event (previous or next).
 All of these methods may return C<undef> if there is no matching
 datetime in the set.
 
+These methods will try to set the returned value 
+to the same time zone as the argument, unless the argument
+has a 'floating' time zone.
+
 =item * map ( sub { ... } )
 
     # example: remove the hour:minute:second information
@@ -991,6 +1022,14 @@ datetime in the set.
         }
     );
 
+    # example: postpone or antecipate events which 
+    #          match datetimes within another set
+    $set = $set2->map(
+        sub {
+            return $_->add( days => 1 ) while $holidays->contains( $_ );
+        }
+    );
+
 This method is the "set" version of Perl "map".
 
 It evaluates a subroutine for each element of
@@ -1006,12 +1045,15 @@ Unlike Perl "map", changing "$_" does not change
 the original set. This means that calling map
 in void context has no effect.
 
-The callback subroutine may not be called immediately.
-Don't count on subroutine side-effects. For example,
+The callback subroutine may be called later in the program,
+due to lazy evaluation.
+So don't count on subroutine side-effects. For example,
 a C<print> inside the subroutine may happen later than you expect.
 
 The callback return value is expected to be within the span of the
 C<previous> and the C<next> element in the original set.
+This is a limitation of the backtracking algorithm used in
+the C<Set::Infinite> library.
 
 For example: given the set C<[ 2001, 2010, 2015 ]>,
 the callback result for the value C<2010> is expected to be
@@ -1039,48 +1081,14 @@ in void context has no effect.
 
 Changing "$_" does change the resulting set.
 
-The callback subroutine may not be called immediately.
-Don't count on subroutine side-effects. For example,
+The callback subroutine may be called later in the program,
+due to lazy evaluation.
+So don't count on subroutine side-effects. For example,
 a C<print> inside the subroutine may happen later than you expect.
 
 =item * iterate ( sub { ... } )
 
-I<Internal method - use "map" or "grep" instead.>
-
-This function apply a callback subroutine to all elements of a set
-and returns the resulting set.
-
-    sub callback {
-        $_[0]->add( hours => 1 );
-    }
-
-    # $set2 elements are one hour after $set elements, and
-    # $set is unchanged
-    $set2 = $set->iterate( \&callback );  
-
-If the callback returns C<undef>, the datetime is removed from the set:
-
-    sub remove_sundays {
-        $_[0] unless $_[0]->day_of_week == 7;
-    }
-
-The callback can be used to postpone or anticipate
-events which collide with datetimes in another set:
-
-    sub after_holiday {
-        $_[0]->add( days => 1 ) while $holidays->contains( $_[0] );
-    }
-
-The callback return value is expected to be within the span of the 
-C<previous> and the C<next> element in the original set. 
-
-For example: given the set C<[ 2001, 2010, 2015 ]>, 
-the callback result for the value C<2010> is expected to be 
-within the span C<[ 2001 .. 2015 ]>.
-
-The callback subroutine may not be called immediately.
-Don't count on subroutine side-effects. For example,
-a C<print> inside the subroutine may happen later than you expect.
+I<deprecated method - please use "map" or "grep" instead.>
 
 =back
 
@@ -1098,7 +1106,7 @@ The API was developed together with Dave Rolsky and the DateTime Community.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003 Flavio Soibelmann Glock. All rights reserved.
+Copyright (c) 2003, 2004 Flavio Soibelmann Glock. All rights reserved.
 This program is free software; you can distribute it and/or
 modify it under the same terms as Perl itself.
 
